@@ -37,10 +37,19 @@ static KeccakPermuteFn *Keccak_Permute_ptr = NULL;
 static KeccakExtractBytesFn *Keccak_ExtractBytes_ptr = NULL;
 static KeccakFastLoopAbsorbFn *Keccak_FastLoopAbsorb_ptr = NULL;
 
+extern struct OQS_SHA3_callbacks sha3_default_callbacks;
+
 static void Keccak_Dispatch(void) {
 // TODO: Simplify this when we have a Windows-compatible AVX2 implementation of SHA3
 #if defined(OQS_DIST_X86_64_BUILD)
 #if defined(OQS_ENABLE_SHA3_xkcp_low_avx2)
+#if defined(OQS_USE_SHA3_AVX512VL)
+	if (OQS_CPU_has_extension(OQS_CPU_EXT_AVX512)) {
+		extern const struct OQS_SHA3_callbacks sha3_avx512vl_callbacks;
+
+		sha3_default_callbacks = sha3_avx512vl_callbacks;
+	}
+#endif
 	if (OQS_CPU_has_extension(OQS_CPU_EXT_AVX2)) {
 		Keccak_Initialize_ptr = &KeccakP1600_Initialize_avx2;
 		Keccak_AddByte_ptr = &KeccakP1600_AddByte_avx2;
@@ -190,7 +199,7 @@ static void keccak_inc_squeeze(uint8_t *h, size_t outlen,
 
 /* SHA3-256 */
 
-void OQS_SHA3_sha3_256(uint8_t *output, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_256(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_256_inc_ctx s;
 	OQS_SHA3_sha3_256_inc_init(&s);
 	OQS_SHA3_sha3_256_inc_absorb(&s, input, inlen);
@@ -198,38 +207,37 @@ void OQS_SHA3_sha3_256(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_256_inc_ctx_release(&s);
 }
 
-void OQS_SHA3_sha3_256_inc_init(OQS_SHA3_sha3_256_inc_ctx *state) {
+static void SHA3_sha3_256_inc_init(OQS_SHA3_sha3_256_inc_ctx *state) {
 	state->ctx = OQS_MEM_aligned_alloc(KECCAK_CTX_ALIGNMENT, KECCAK_CTX_BYTES);
-	if (state->ctx == NULL) {
-		exit(111);
-	}
+
+	OQS_EXIT_IF_NULLPTR(state->ctx, "SHA3");
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
-void OQS_SHA3_sha3_256_inc_absorb(OQS_SHA3_sha3_256_inc_ctx *state, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_256_inc_absorb(OQS_SHA3_sha3_256_inc_ctx *state, const uint8_t *input, size_t inlen) {
 	keccak_inc_absorb((uint64_t *)state->ctx, OQS_SHA3_SHA3_256_RATE, input, inlen);
 }
 
-void OQS_SHA3_sha3_256_inc_finalize(uint8_t *output, OQS_SHA3_sha3_256_inc_ctx *state) {
+static void SHA3_sha3_256_inc_finalize(uint8_t *output, OQS_SHA3_sha3_256_inc_ctx *state) {
 	keccak_inc_finalize((uint64_t *)state->ctx, OQS_SHA3_SHA3_256_RATE, 0x06);
 	keccak_inc_squeeze(output, 32, (uint64_t *)state->ctx, OQS_SHA3_SHA3_256_RATE);
 }
 
-void OQS_SHA3_sha3_256_inc_ctx_release(OQS_SHA3_sha3_256_inc_ctx *state) {
-	OQS_MEM_aligned_free(state->ctx);
+static void SHA3_sha3_256_inc_ctx_release(OQS_SHA3_sha3_256_inc_ctx *state) {
+	OQS_MEM_aligned_secure_free(state->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_256_inc_ctx_clone(OQS_SHA3_sha3_256_inc_ctx *dest, const OQS_SHA3_sha3_256_inc_ctx *src) {
+static void SHA3_sha3_256_inc_ctx_clone(OQS_SHA3_sha3_256_inc_ctx *dest, const OQS_SHA3_sha3_256_inc_ctx *src) {
 	memcpy(dest->ctx, src->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_256_inc_ctx_reset(OQS_SHA3_sha3_256_inc_ctx *state) {
+static void SHA3_sha3_256_inc_ctx_reset(OQS_SHA3_sha3_256_inc_ctx *state) {
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
 /* SHA3-384 */
 
-void OQS_SHA3_sha3_384(uint8_t *output, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_384(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_384_inc_ctx s;
 	OQS_SHA3_sha3_384_inc_init(&s);
 	OQS_SHA3_sha3_384_inc_absorb(&s, input, inlen);
@@ -237,38 +245,35 @@ void OQS_SHA3_sha3_384(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_384_inc_ctx_release(&s);
 }
 
-void OQS_SHA3_sha3_384_inc_init(OQS_SHA3_sha3_384_inc_ctx *state) {
+static void SHA3_sha3_384_inc_init(OQS_SHA3_sha3_384_inc_ctx *state) {
 	state->ctx = OQS_MEM_aligned_alloc(KECCAK_CTX_ALIGNMENT, KECCAK_CTX_BYTES);
-	if (state->ctx == NULL) {
-		exit(111);
-	}
+	OQS_EXIT_IF_NULLPTR(state->ctx, "SHA3");
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
-
-void OQS_SHA3_sha3_384_inc_absorb(OQS_SHA3_sha3_384_inc_ctx *state, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_384_inc_absorb(OQS_SHA3_sha3_384_inc_ctx *state, const uint8_t *input, size_t inlen) {
 	keccak_inc_absorb((uint64_t *)state->ctx, OQS_SHA3_SHA3_384_RATE, input, inlen);
 }
 
-void OQS_SHA3_sha3_384_inc_finalize(uint8_t *output, OQS_SHA3_sha3_384_inc_ctx *state) {
+static void SHA3_sha3_384_inc_finalize(uint8_t *output, OQS_SHA3_sha3_384_inc_ctx *state) {
 	keccak_inc_finalize((uint64_t *)state->ctx, OQS_SHA3_SHA3_384_RATE, 0x06);
 	keccak_inc_squeeze(output, 48, (uint64_t *)state->ctx, OQS_SHA3_SHA3_384_RATE);
 }
 
-void OQS_SHA3_sha3_384_inc_ctx_release(OQS_SHA3_sha3_384_inc_ctx *state) {
-	OQS_MEM_aligned_free(state->ctx);
+static void SHA3_sha3_384_inc_ctx_release(OQS_SHA3_sha3_384_inc_ctx *state) {
+	OQS_MEM_aligned_secure_free(state->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_384_inc_ctx_clone(OQS_SHA3_sha3_384_inc_ctx *dest, const OQS_SHA3_sha3_384_inc_ctx *src) {
+static void SHA3_sha3_384_inc_ctx_clone(OQS_SHA3_sha3_384_inc_ctx *dest, const OQS_SHA3_sha3_384_inc_ctx *src) {
 	memcpy(dest->ctx, src->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_384_inc_ctx_reset(OQS_SHA3_sha3_384_inc_ctx *state) {
+static void SHA3_sha3_384_inc_ctx_reset(OQS_SHA3_sha3_384_inc_ctx *state) {
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
 /* SHA3-512 */
 
-void OQS_SHA3_sha3_512(uint8_t *output, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_512(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_512_inc_ctx s;
 	OQS_SHA3_sha3_512_inc_init(&s);
 	OQS_SHA3_sha3_512_inc_absorb(&s, input, inlen);
@@ -276,38 +281,36 @@ void OQS_SHA3_sha3_512(uint8_t *output, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_sha3_512_inc_ctx_release(&s);
 }
 
-void OQS_SHA3_sha3_512_inc_init(OQS_SHA3_sha3_512_inc_ctx *state) {
+static void SHA3_sha3_512_inc_init(OQS_SHA3_sha3_512_inc_ctx *state) {
 	state->ctx = OQS_MEM_aligned_alloc(KECCAK_CTX_ALIGNMENT, KECCAK_CTX_BYTES);
-	if (state->ctx == NULL) {
-		exit(111);
-	}
+	OQS_EXIT_IF_NULLPTR(state->ctx, "SHA3");
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
-void OQS_SHA3_sha3_512_inc_absorb(OQS_SHA3_sha3_512_inc_ctx *state, const uint8_t *input, size_t inlen) {
+static void SHA3_sha3_512_inc_absorb(OQS_SHA3_sha3_512_inc_ctx *state, const uint8_t *input, size_t inlen) {
 	keccak_inc_absorb((uint64_t *)state->ctx, OQS_SHA3_SHA3_512_RATE, input, inlen);
 }
 
-void OQS_SHA3_sha3_512_inc_finalize(uint8_t *output, OQS_SHA3_sha3_512_inc_ctx *state) {
+static void SHA3_sha3_512_inc_finalize(uint8_t *output, OQS_SHA3_sha3_512_inc_ctx *state) {
 	keccak_inc_finalize((uint64_t *)state->ctx, OQS_SHA3_SHA3_512_RATE, 0x06);
 	keccak_inc_squeeze(output, 64, (uint64_t *)state->ctx, OQS_SHA3_SHA3_512_RATE);
 }
 
-void OQS_SHA3_sha3_512_inc_ctx_release(OQS_SHA3_sha3_512_inc_ctx *state) {
-	OQS_MEM_aligned_free(state->ctx);
+static void SHA3_sha3_512_inc_ctx_release(OQS_SHA3_sha3_512_inc_ctx *state) {
+	OQS_MEM_aligned_secure_free(state->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_512_inc_ctx_clone(OQS_SHA3_sha3_512_inc_ctx *dest, const OQS_SHA3_sha3_512_inc_ctx *src) {
+static void SHA3_sha3_512_inc_ctx_clone(OQS_SHA3_sha3_512_inc_ctx *dest, const OQS_SHA3_sha3_512_inc_ctx *src) {
 	memcpy(dest->ctx, src->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_sha3_512_inc_ctx_reset(OQS_SHA3_sha3_512_inc_ctx *state) {
+static void SHA3_sha3_512_inc_ctx_reset(OQS_SHA3_sha3_512_inc_ctx *state) {
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
 /* SHAKE128 */
 
-void OQS_SHA3_shake128(uint8_t *output, size_t outlen, const uint8_t *input, size_t inlen) {
+static void SHA3_shake128(uint8_t *output, size_t outlen, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_shake128_inc_ctx s;
 	OQS_SHA3_shake128_inc_init(&s);
 	OQS_SHA3_shake128_inc_absorb(&s, input, inlen);
@@ -318,41 +321,39 @@ void OQS_SHA3_shake128(uint8_t *output, size_t outlen, const uint8_t *input, siz
 
 /* SHAKE128 incremental */
 
-void OQS_SHA3_shake128_inc_init(OQS_SHA3_shake128_inc_ctx *state) {
+static void SHA3_shake128_inc_init(OQS_SHA3_shake128_inc_ctx *state) {
 	state->ctx = OQS_MEM_aligned_alloc(KECCAK_CTX_ALIGNMENT, KECCAK_CTX_BYTES);
-	if (state->ctx == NULL) {
-		exit(111);
-	}
+	OQS_EXIT_IF_NULLPTR(state->ctx, "SHA3");
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
-void OQS_SHA3_shake128_inc_absorb(OQS_SHA3_shake128_inc_ctx *state, const uint8_t *input, size_t inlen) {
+static void SHA3_shake128_inc_absorb(OQS_SHA3_shake128_inc_ctx *state, const uint8_t *input, size_t inlen) {
 	keccak_inc_absorb((uint64_t *)state->ctx, OQS_SHA3_SHAKE128_RATE, input, inlen);
 }
 
-void OQS_SHA3_shake128_inc_finalize(OQS_SHA3_shake128_inc_ctx *state) {
+static void SHA3_shake128_inc_finalize(OQS_SHA3_shake128_inc_ctx *state) {
 	keccak_inc_finalize((uint64_t *)state->ctx, OQS_SHA3_SHAKE128_RATE, 0x1F);
 }
 
-void OQS_SHA3_shake128_inc_squeeze(uint8_t *output, size_t outlen, OQS_SHA3_shake128_inc_ctx *state) {
+static void SHA3_shake128_inc_squeeze(uint8_t *output, size_t outlen, OQS_SHA3_shake128_inc_ctx *state) {
 	keccak_inc_squeeze(output, outlen, (uint64_t *)state->ctx, OQS_SHA3_SHAKE128_RATE);
 }
 
-void OQS_SHA3_shake128_inc_ctx_clone(OQS_SHA3_shake128_inc_ctx *dest, const OQS_SHA3_shake128_inc_ctx *src) {
+static void SHA3_shake128_inc_ctx_clone(OQS_SHA3_shake128_inc_ctx *dest, const OQS_SHA3_shake128_inc_ctx *src) {
 	memcpy(dest->ctx, src->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_shake128_inc_ctx_release(OQS_SHA3_shake128_inc_ctx *state) {
-	OQS_MEM_aligned_free(state->ctx);
+static void SHA3_shake128_inc_ctx_release(OQS_SHA3_shake128_inc_ctx *state) {
+	OQS_MEM_aligned_secure_free(state->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_shake128_inc_ctx_reset(OQS_SHA3_shake128_inc_ctx *state) {
+static void SHA3_shake128_inc_ctx_reset(OQS_SHA3_shake128_inc_ctx *state) {
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
 /* SHAKE256 */
 
-void OQS_SHA3_shake256(uint8_t *output, size_t outlen, const uint8_t *input, size_t inlen) {
+static void SHA3_shake256(uint8_t *output, size_t outlen, const uint8_t *input, size_t inlen) {
 	OQS_SHA3_shake256_inc_ctx s;
 	OQS_SHA3_shake256_inc_init(&s);
 	OQS_SHA3_shake256_inc_absorb(&s, input, inlen);
@@ -363,34 +364,72 @@ void OQS_SHA3_shake256(uint8_t *output, size_t outlen, const uint8_t *input, siz
 
 /* SHAKE256 incremental */
 
-void OQS_SHA3_shake256_inc_init(OQS_SHA3_shake256_inc_ctx *state) {
+static void SHA3_shake256_inc_init(OQS_SHA3_shake256_inc_ctx *state) {
 	state->ctx = OQS_MEM_aligned_alloc(KECCAK_CTX_ALIGNMENT, KECCAK_CTX_BYTES);
-	if (state->ctx == NULL) {
-		exit(111);
-	}
+	OQS_EXIT_IF_NULLPTR(state->ctx, "SHA3");
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
 
-void OQS_SHA3_shake256_inc_absorb(OQS_SHA3_shake256_inc_ctx *state, const uint8_t *input, size_t inlen) {
+static void SHA3_shake256_inc_absorb(OQS_SHA3_shake256_inc_ctx *state, const uint8_t *input, size_t inlen) {
 	keccak_inc_absorb((uint64_t *)state->ctx, OQS_SHA3_SHAKE256_RATE, input, inlen);
 }
 
-void OQS_SHA3_shake256_inc_finalize(OQS_SHA3_shake256_inc_ctx *state) {
+static void SHA3_shake256_inc_finalize(OQS_SHA3_shake256_inc_ctx *state) {
 	keccak_inc_finalize((uint64_t *)state->ctx, OQS_SHA3_SHAKE256_RATE, 0x1F);
 }
 
-void OQS_SHA3_shake256_inc_squeeze(uint8_t *output, size_t outlen, OQS_SHA3_shake256_inc_ctx *state) {
+static void SHA3_shake256_inc_squeeze(uint8_t *output, size_t outlen, OQS_SHA3_shake256_inc_ctx *state) {
 	keccak_inc_squeeze(output, outlen, state->ctx, OQS_SHA3_SHAKE256_RATE);
 }
 
-void OQS_SHA3_shake256_inc_ctx_release(OQS_SHA3_shake256_inc_ctx *state) {
-	OQS_MEM_aligned_free(state->ctx);
+static void SHA3_shake256_inc_ctx_release(OQS_SHA3_shake256_inc_ctx *state) {
+	OQS_MEM_aligned_secure_free(state->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_shake256_inc_ctx_clone(OQS_SHA3_shake256_inc_ctx *dest, const OQS_SHA3_shake256_inc_ctx *src) {
+static void SHA3_shake256_inc_ctx_clone(OQS_SHA3_shake256_inc_ctx *dest, const OQS_SHA3_shake256_inc_ctx *src) {
 	memcpy(dest->ctx, src->ctx, KECCAK_CTX_BYTES);
 }
 
-void OQS_SHA3_shake256_inc_ctx_reset(OQS_SHA3_shake256_inc_ctx *state) {
+static void SHA3_shake256_inc_ctx_reset(OQS_SHA3_shake256_inc_ctx *state) {
 	keccak_inc_reset((uint64_t *)state->ctx);
 }
+
+struct OQS_SHA3_callbacks sha3_default_callbacks = {
+	SHA3_sha3_256,
+	SHA3_sha3_256_inc_init,
+	SHA3_sha3_256_inc_absorb,
+	SHA3_sha3_256_inc_finalize,
+	SHA3_sha3_256_inc_ctx_release,
+	SHA3_sha3_256_inc_ctx_reset,
+	SHA3_sha3_256_inc_ctx_clone,
+	SHA3_sha3_384,
+	SHA3_sha3_384_inc_init,
+	SHA3_sha3_384_inc_absorb,
+	SHA3_sha3_384_inc_finalize,
+	SHA3_sha3_384_inc_ctx_release,
+	SHA3_sha3_384_inc_ctx_reset,
+	SHA3_sha3_384_inc_ctx_clone,
+	SHA3_sha3_512,
+	SHA3_sha3_512_inc_init,
+	SHA3_sha3_512_inc_absorb,
+	SHA3_sha3_512_inc_finalize,
+	SHA3_sha3_512_inc_ctx_release,
+	SHA3_sha3_512_inc_ctx_reset,
+	SHA3_sha3_512_inc_ctx_clone,
+	SHA3_shake128,
+	SHA3_shake128_inc_init,
+	SHA3_shake128_inc_absorb,
+	SHA3_shake128_inc_finalize,
+	SHA3_shake128_inc_squeeze,
+	SHA3_shake128_inc_ctx_release,
+	SHA3_shake128_inc_ctx_clone,
+	SHA3_shake128_inc_ctx_reset,
+	SHA3_shake256,
+	SHA3_shake256_inc_init,
+	SHA3_shake256_inc_absorb,
+	SHA3_shake256_inc_finalize,
+	SHA3_shake256_inc_squeeze,
+	SHA3_shake256_inc_ctx_release,
+	SHA3_shake256_inc_ctx_clone,
+	SHA3_shake256_inc_ctx_reset,
+};
