@@ -94,7 +94,7 @@ int ReadHex(FILE *infile, unsigned char *a, unsigned long Length, char *str) {
 		a[0] = 0x00;
 		return 1;
 	}
-	memset(a, 0x00, Length);
+	OQS_MEM_cleanse(a, Length);
 	started = 0;
 	if (FindMarker(infile, str))
 		while ((ch = fgetc(infile)) != EOF) {
@@ -734,7 +734,7 @@ static void TEST_SIG_STFL_randombytes(uint8_t *random_array, size_t bytes_to_rea
 	// We can't make direct calls to the system randombytes on some platforms,
 	// so we have to swap out the OQS_randombytes provider.
 
-	OQS_randombytes_switch_algorithm("system");
+	(void)OQS_randombytes_switch_algorithm("system");
 	OQS_randombytes(random_array, bytes_to_read);
 	OQS_randombytes_custom_algorithm(&TEST_SIG_STFL_randombytes);
 
@@ -1063,8 +1063,13 @@ static OQS_STATUS update_test_result( OQS_STATUS rc, int xmss_or_lms) {
 }
 
 int main(int argc, char **argv) {
+	OQS_STATUS  rc = OQS_ERROR, rc1 = OQS_ERROR;
 	OQS_init();
-	oqs_fstore_init();
+	rc = oqs_fstore_init();
+	if (rc != OQS_SUCCESS) {
+		OQS_destroy();
+		return EXIT_FAILURE;
+	}
 
 	printf("Testing stateful signature algorithms using liboqs version %s\n", OQS_version());
 
@@ -1141,10 +1146,14 @@ int main(int argc, char **argv) {
 #ifdef OQS_ENABLE_TEST_CONSTANT_TIME
 	OQS_randombytes_custom_algorithm(&TEST_SIG_STFL_randombytes);
 #else
-	OQS_randombytes_switch_algorithm("system");
+	rc = OQS_randombytes_switch_algorithm("system");
+	if (rc != OQS_SUCCESS) {
+		printf("Could not generate random data with system RNG\n");
+		OQS_destroy();
+		return EXIT_FAILURE;
+	}
 #endif
 
-	OQS_STATUS  rc = OQS_ERROR, rc1 = OQS_ERROR;
 	int exit_status = EXIT_SUCCESS;
 
 #if OQS_USE_PTHREADS_IN_TESTS
